@@ -1,6 +1,8 @@
+import time
+
 from typing import Tuple, List, Union
 from modi.module.module import Module
-from modi.util.msgutil import parse_message, parse_data
+from modi.util.message_util import parse_message, parse_data
 
 
 class OutputModule(Module):
@@ -72,10 +74,12 @@ class OutputModule(Module):
             destination_id,
             property_type,
             property_values,
-            property_data_type)
+            property_data_type
+        )
 
         for message in messages:
             self._conn.send(message)
+            time.sleep(0.01)
 
     @staticmethod
     def _validate_property(nb_values: int, value_range: Tuple = None):
